<?php
/* 
Description: Core Library EMail API functions
 
Copyright 2014 Malcolm Shergold

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

if (!class_exists('StageShowLibMailer')) 
{
	require_once ABSPATH . WPINC . '/class-phpmailer.php';
	require_once ABSPATH . WPINC . '/class-smtp.php';
	
	define('STAGESHOWLIB_FILENAME_EMAILLOG', 'EMailLog.log');
	
	class StageShowLibMailer extends PHPMailer
	{
		var $lastError = '';
		
		function preSend()
		{
			// Force the From name
			$this->FromName = $this->StageShow_FromName;
			
			$status = parent::preSend();

			if ($this->SMTPDebug)
			{
				echo "<br>Called PHPMailer::preSend() - Return status=$status<br>\n";
			}
			
			return $status;
		}
		
		public function postSend()
		{
			if ($this->SMTPDebug)
			{
				echo "Mailer: ".$this->Mailer."<br>\n";
			}
			
			if ($this->EMailLogPath != '')
			{
				$debugMessage = "Sent EMail using ".$this->Mailer." Mailer\n\n";
				$debugMessage .= "MIMEHeader:\n".$this->MIMEHeader."\n\n";
				$debugMessage .= "MIMEBody:\n".$this->MIMEBody."\n\n";

                $this->LogEMail($this->EMailLogPath, $debugMessage);
			}
			
			$status = parent::postSend();
			
			if ($this->SMTPDebug)
			{
				echo "Called PHPMailer::PostSend() - Return status=$status<br>\n";
			}
			
			return $status;
		}
		
		function LogEMail($LogsFolder, $DebugMessage)
		{
			$LogNotifyFile = STAGESHOWLIB_FILENAME_EMAILLOG;				
			$logFileObj = new StageShowLibLogFileClass($LogsFolder);
			$logFileObj->StampedLogToFile($LogNotifyFile, $DebugMessage, StageShowLibDBaseClass::ForAppending);
		}

	}
}

if (!class_exists('StageShowLibEMailAPIClass')) 
{
	class StageShowLibEMailAPIClass // Define class
	{	
		var $parentObj;
		var $adminEMail;
		
		function __construct($ourParentObj)	
		{
			$this->parentObj = $ourParentObj;	
			$this->adminEMail = get_option('admin_email');		
		}
		
		function createPHPMailerObj($SMTPDebug, $EMailLogPath)
		{
			global $phpmailer;
			$phpmailer = new StageShowLibMailer( true );		
			$phpmailer->SMTPDebug = $SMTPDebug;
			$phpmailer->EMailLogPath = $EMailLogPath;
		}
		
		function AddImage($imageObj)
		{
			return false;
		}
		
		function AddAttachment($filePath)
		{
		}
		
		function sendMail($to, $from, $subject, $content, $content2 = '', $headers = '')
		{
			global $phpmailer;
			
			$SMTPDebug = $this->parentObj->getDbgOption('Dev_ShowEMailMsgs');
			$EMailLogPath = $this->parentObj->getDbgOption('Dev_LogEMailMsgs') ? $this->parentObj->getOption('LogsFolderPath') : '';
			$this->createPHPMailerObj($SMTPDebug, $EMailLogPath);
			
			$replyTo = $from;
			
			// Define the email headers - separated with \r\n
			if (strlen($headers) > 0) $headers .= "\r\n";
			$headers .= "From: $from";					
			$headers .= "\r\nReply-To: $replyTo";	

			if ($SMTPDebug)
			{
				// FUNCTIONALITY: General - Echo EMail when Dev_ShowEMailMsgs selected - Body Encoded with htmlspecialchars
				echo "To:<br>\n";
				echo htmlspecialchars($to);
				echo "<br>\n<br>\n";
				echo "Headers:<br>\n";
				echo str_replace("\r\n", "<br>\r\n", htmlspecialchars($headers));
				echo "<br>\n<br>\n";
				echo "Message:<br>\n";
				echo htmlspecialchars($content);
				echo "<br>\n<br>\n";
			}
					
			$bracket_pos = strpos( $from, '<' );
			if ( $bracket_pos !== false ) 
				{
				// Text before the bracketed email is the "From" name.
				if ( $bracket_pos > 0 ) 
				{
					$from_name = substr( $from, 0, $bracket_pos - 1 );
					$from_name = str_replace( '"', '', $from_name );
					$from_name = trim( $from_name );
				}

				$from_email = substr( $from, $bracket_pos + 1 );
				$from_email = str_replace( '>', '', $from_email );
				$from_email = trim( $from_email );
			} 
			else
			{
				$from_email = trim( $from );
				$from_name = '';
			}
			$phpmailer->StageShow_From = $from_email;
			$phpmailer->StageShow_FromName = $from_name;
									
			// Register a function to get any EMail errors
			add_action('wp_mail_failed', array(&$this, 'Get_EMailErrors'));
			
			// FUNCTIONALITY: General - Send EMail
			$toList = explode(';', $to);
			if (!$this->parentObj->getDbgOption('Dev_BlockEMailSend'))
			{
				if (!wp_mail($toList, $subject, $content, $headers))
				{
					return $this->lastError;
				}
			}
			else
			{
				echo "<br><strong>Sending of EMails Blocked </strong><br><br>\n";	
				echo "EMail to: ".htmlspecialchars($to)." <br>\n";	
				echo "Subject: $subject <br>\n";
				$headersDump = StageShowLibUtilsClass::print_r($headers, '$headers', true);
				$headersDump = str_replace("\n", "<br>\n", htmlspecialchars(str_replace("<br>", "", $headersDump)));
				echo $headersDump;
			}

			return 'OK';
		}	
		
		function Get_EMailErrors($excp)
		{
			$errMsg = $excp->get_error_message();
			$this->lastError = $errMsg;
		}

	}
}

?>